import java_cup.runtime.Symbol;
import java_cup.runtime.*;

%% 

/* 
   The name of the class JFlex will create will be Lexer.
   Will write the code to the file Lexer.java. 
*/
%class Lexer

/*
  The current line number can be accessed with the variable yyline
  and the current column number with the variable yycolumn.
*/
%line
%column
    
/* 
   Will switch to a CUP compatibility mode to interface with a CUP
   generated parser.
*/
%cup
   
/*
  Declarations
   
  Code between %{ and %}, both of which must be at the beginning of a
  line, will be copied letter to letter into the lexer class source.
  Here you declare member variables and functions that are used inside
  scanner actions.  
*/

%{   
    /* To create a new java_cup.runtime.Symbol with information about
       the current token, the token will have no value in this
       case. */
    private Symbol symbol(int type) {
        return new Symbol(type, yyline, yycolumn);
    }
    
    /* Also creates a new java_cup.runtime.Symbol with information
       about the current token, but this object has a value. */
    private Symbol symbol(int type, Object value) {
        return new Symbol(type, yyline, yycolumn, value);
    }
%}

/*
  Macro Declarations
  
  These declarations are regular expressions that will be used latter
  in the Lexical Rules Section.  
*/
   
/* A line terminator is a \r (carriage return), \n (line feed), or
   \r\n. */
LineTerminator = \r|\n|\r\n
   
/* White space is a line terminator, space, tab, or line feed. */
WhiteSpace     = {LineTerminator} | [ \t\f]
   
/*Number*/

number = ("+"|"-")?[0-9]+("."[0-9]+(e("+"|"-")[0-9]+)?)?

/*Name*/


name = [a-zA-Z_][a-z0-9A-Z_]*

stringa = [a-zA-Z_][a-zA-Z_]*


%%


	"(" {return new Symbol(sym.LPAREN,"(");}
	")" { return new Symbol(sym.RPAREN,")");}
	"[" { return new Symbol(sym.LBRACK,"[");}
	"]" { return new Symbol(sym.RBRACK,"]");}
	"{" { return new Symbol(sym.LCURLY,"{");}
	"}" { return new Symbol(sym.RCURLY,"}");}
	":" { return new Symbol(sym.COLON,":");}
	"::" { return new Symbol(sym.DCOLON,"::");}
	"," {return new Symbol(sym.COMMA,",");}
	"=" { return new Symbol(sym.EQUAL,"=");}
	"<>" { return new Symbol(sym.NEQUAL,"<>");}
	"<" { return new Symbol(sym.LT,"<");}
	">" { return new Symbol(sym.GT,">");}
	"<=" { return new Symbol(sym.LE,"<=");}
	">=" { return new Symbol(sym.GE,">=");}
	"->" { return new Symbol(sym.RARROW,"->");}
	".." { return new Symbol(sym.DOTDOT,"..");}
	"." { return new Symbol(sym.DOT,".");}
	";" { return new Symbol(sym.SEMICOL,";");}
	"|" {return new Symbol(sym.BAR,"|");}
	"@" { return new Symbol(sym.ATSIGN,"@");}
	"+" { return new Symbol(sym.PLUS,"+");}
	"-" { return new Symbol(sym.MINUS,"-");}
	"*" { return new Symbol(sym.MULT,"*");}
	"/" { return new Symbol(sym.DIVIDE,"/");}
	
	"package" { return new Symbol(sym.PACKAGE,"package");}
	"endpackage" { return new Symbol(sym.ENDPACKAGE,"endPackage");}
	"def" { return new Symbol(sym.DEF,"def");}
	"context" { return new Symbol(sym.CONTEXT,"context");}
	"pre" { return new Symbol(sym.PRE,"pre");}
	"post" { return new Symbol(sym.POST,"post");}
	"inv" { return new Symbol(sym.INV,"inv");}
	"implies" { return new Symbol(sym.IMPLIES,"implies");}
	"not" { return new Symbol(sym.NOT,"not");}
	"or" { return new Symbol(sym.OR,"or");}
	"xor" { return new Symbol(sym.XOR,"xor");}
	"and" { return new Symbol(sym.AND,"and");}
	"in" { return new Symbol(sym.IN,"in");}
	"let" { return new Symbol(sym.LET,"let");}
	"if" { return new Symbol(sym.IF,"if");}
	"then" { return new Symbol(sym.THEN,"then");}
	"else" { return new Symbol(sym.ELSE,"else");}
	"endif" { return new Symbol(sym.ENDIF,"endif");}
	"set" | "Set" { return new Symbol(sym.SET,"set");}
	"bag" { return new Symbol(sym.BAG,"bag");}
	"sequence" { return new Symbol(sym.SEQUENCE,"sequence");}
	"collection" { return new Symbol(sym.COLLECTION,"collection");}
	
	
    {number}      { return symbol(sym.NUMBER,  new Double(yytext())); }
    {name}       {return symbol(sym.NAME, yytext());}
    {stringa}       { return symbol(sym.STRING, yytext());}
     
    {WhiteSpace}       {}   


/* No token was found for the input so through an error.  Print out an
   Illegal character message with the illegal character that was found. */
[^]                    { throw new Error("Illegal character <"+yytext()+">"); }


  